
#include "GRAM.h"

GRAM::GRAM()
    : image(NULL),
      dirtyBytes(NULL)
{}

void GRAM::init(INT32 size, INT32 location, INT32 bitWidth)
{
    this->size = size;
    this->location = location;
    this->bitWidth = bitWidth;
    trimmer = (1 << bitWidth) -1;
    image = new INT32[size];
    dirtyBytes = new BOOL[size];
}

GRAM::~GRAM() {
    if (image != NULL)
        delete[] image;
    if (dirtyBytes != NULL)
        delete[] dirtyBytes;
}

void GRAM::reset() {
    dirtyRAM = TRUE;
    for (INT32 i = 0; i < size; i++) {
        image[i] = 0;
        dirtyBytes[i] = TRUE;
    }
}

INT32 GRAM::getLocation() {
    return location;
}

INT32 GRAM::peek(INT32 location) {
    return image[location-this->location];
}

void GRAM::poke(INT32 location, INT32 value) {
    value = (value & trimmer);
    if (image[location-this->location] != value) {
        image[location-this->location] = value;
        dirtyBytes[location-this->location] = TRUE;
        dirtyRAM = TRUE;
    }
}

void GRAM::markClean() {
    if (!dirtyRAM)
        return;

    for (INT32 i = 0; i < size; i++)
        dirtyBytes[i] = FALSE;
    dirtyRAM = FALSE;
}

BOOL GRAM::isDirty() {
    return dirtyRAM;
}

BOOL GRAM::isDirty(INT32 location) {
    return dirtyBytes[location-this->location];
}

INT32 GRAM::getSize() {
    return size;
}

