
#include "ECSKeyboard.h"

const INT32 ECSKeyboard::mappings[48] = {
     0,  1,  2,  3,  4,  5,
     8,  9, 10, 11, 12, 13,
    16, 17, 18, 19, 20, 21,
    24, 25, 26, 27, 28, 29,
    32, 33, 34, 35, 36, 37,
    40, 41, 42, 43, 44, 45,
    48, 49, 50, 51, 52, 53,
    57, 58, 59, 60, 61, 62
};

ECSKeyboard::ECSKeyboard() {
    for (INT32 i = 0; i < 64; i++)
        inputSignals[i] = NULL;
}

void ECSKeyboard::setInputDevice(InputDevice* inputDevice) {
    this->inputDevice = inputDevice;

    InputSignal* defaultSignal;
    for (INT32 i = 0; i < 48; i++) {
        inputDevice->getDefaultConfiguration(2,//InputDevice::ID_ECS_KEYBOARD,
                i, &defaultSignal);
        this->inputSignals[mappings[i]] = defaultSignal;
    }
}

void ECSKeyboard::reset() {
    for (INT32 i = 0; i < 8; i++)
        rowInputValues[i] = 0;
}

void ECSKeyboard::poll() {
    for (INT32 row = 0; row < 8; row++) {
        rowInputValues[row] = 0;
        if (inputDevice->getSignalValue(inputSignals[row]) ==
                InputDevice::SIGNAL_ON)
            rowInputValues[row] |= 0x01;
        if (inputDevice->getSignalValue(inputSignals[row+8]) ==
                InputDevice::SIGNAL_ON)
            rowInputValues[row] |= 0x02;
        if (inputDevice->getSignalValue(inputSignals[row+16]) ==
                InputDevice::SIGNAL_ON)
            rowInputValues[row] |= 0x04;
        if (inputDevice->getSignalValue(inputSignals[row+24]) ==
                InputDevice::SIGNAL_ON)
            rowInputValues[row] |= 0x08;
        if (inputDevice->getSignalValue(inputSignals[row+32]) ==
                InputDevice::SIGNAL_ON)
            rowInputValues[row] |= 0x10;
        if (inputDevice->getSignalValue(inputSignals[row+40]) ==
                InputDevice::SIGNAL_ON)
            rowInputValues[row] |= 0x20;
        if (inputDevice->getSignalValue(inputSignals[row+48]) ==
                InputDevice::SIGNAL_ON)
            rowInputValues[row] |= 0x40;
        if (inputDevice->getSignalValue(inputSignals[row+56]) ==
                InputDevice::SIGNAL_ON)
            rowInputValues[row] |= 0x80;
    }
}

INT32 ECSKeyboard::getInputValue() {
    INT32 inputValue = 0;
    INT32 rowMask = 1;
    for (INT32 row = 0; row < 8; row++)  {
        if ((rowsToScan & rowMask) != 0) {
            rowMask = rowMask << 1;
            continue;
        }
        inputValue |= rowInputValues[row];

        rowMask = rowMask << 1;
    }

    return (0xFF ^ inputValue);
}

void ECSKeyboard::setOutputValue(INT32 value) {
    this->rowsToScan = value;
}

INT32 ECSKeyboard::getControlCount() {
    return 48;
}

void ECSKeyboard::setControlInputSignal(INT32 controlNum,
        InputSignal* inputSignal)
{
    inputSignals[mappings[controlNum]] = inputSignal;
}

void ECSKeyboard::getControlInputSignal(INT32 controlNum,
        InputSignal** inputSignal)
{
    (*inputSignal) = inputSignals[mappings[controlNum]];
}

void ECSKeyboard::getControlDescription(INT32 controlNum, CHAR* desc) {
    //nothing for now
}

/*
    private final static String[] descriptionKeys = {
            "InputECSLeft",
            "InputECSComma",
            "InputECSN",
            "InputECSV",
            "InputECSX",
            "InputECSSpace",
            "InputECSPeriod",
            "InputECSM",
            "InputECSB",
            "InputECSC",
            "InputECSZ",
            "InputECSDown",
            "InputECSSemiColon",
            "InputECSK",
            "InputECSH",
            "InputECSF",
            "InputECSS",
            "InputECSUp",
            "InputECSP",
            "InputECSI",
            "InputECSY",
            "InputECSR",
            "InputECSW",
            "InputECSQ",
            "InputECSEscape",
            "InputECS9",
            "InputECS7",
            "InputECS5",
            "InputECS3",
            "InputECS1",
            "InputECS0",
            "InputECS8",
            "InputECS6",        
            "InputECS4",
            "InputECS2",
            "InputECSRight",
            "InputECSEnter",    
            "InputECSO",
            "InputECSU",
            "InputECST",
            "InputECSE",
            "InputECSControl",
            "InputECSL",
            "InputECSJ",
            "InputECSG",
            "InputECSD",
            "InputECSA",
            "InputECSShift",
        };
*/
