
#include <windows.h>
#include <iostream>
#include <commctrl.h>
#include "Xanadu.h"
//#include "XanaduMainWindow.h"
#include "CartridgeType.h"
#include "CartridgeTypeFactory.h"
#include "CRC32.h"
#include "Win32PlugIn.h"
#include "ROM.h"

using namespace std;

const UINT32 Xanadu::CRC_EXECUTIVE_ROM = 0xCBCE86F7;
const UINT32 Xanadu::CRC_GROM          = 0x683A4158;
const UINT32 Xanadu::CRC_ECS_ROM       = 0xEA790A06;
const UINT32 Xanadu::CRC_IVOICE_ROM    = 0x0DE7579D;
const CHAR* Xanadu::FILENAME_EXECUTIVE_ROM = "exec.bin";
const CHAR* Xanadu::FILENAME_GROM          = "grom.bin";
const CHAR* Xanadu::FILENAME_ECS_ROM       = "ecs.bin";
const CHAR* Xanadu::FILENAME_IVOICE_ROM    = "ivoice.bin";

INT32* Xanadu::execImage = NULL;
INT32* Xanadu::gromImage = NULL;
INT32* Xanadu::ecsImage = NULL;
INT32* Xanadu::ivoiceImage = NULL;

vector<CartridgeFile*> Xanadu::cartridgeFileList;

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, 
        LPSTR lpCmdLine, int nCmdShow)
/*
INT32 main(INT32 argc, CHAR* argv[])
*/
{
    Xanadu xanadu;
    return xanadu.run(lpCmdLine);
    //return xanadu.run((argc > 1 ? argv[1] : ""));
}

Xanadu::Xanadu()
{
    execImage = NULL;
    gromImage = NULL;
    ecsImage = NULL;
    ivoiceImage = NULL;
    kinfos.gameList = NULL;
}

INT32 Xanadu::run(CHAR* lpCmdLine) {
    CHAR currentDir[MAX_PATH];
    HMODULE hInstance = GetModuleHandle(NULL);
    GetModuleFileName(hInstance, currentDir, MAX_PATH);
    CloseHandle(hInstance);
    INT32 len = strlen(currentDir);
    while (currentDir[len] != '\\')
        len--;
    currentDir[len+1] = NULL;
    CHAR nextFile[MAX_PATH];

    //TODO: verify that the executive rom file exists
    strcpy(nextFile, currentDir);
    strcat(nextFile, "\\");
    strcat(nextFile, FILENAME_EXECUTIVE_ROM);

    //verify the CRC of the executive rom file
    UINT32 crc = CRC32::getCrc(nextFile);
    if (crc != CRC_EXECUTIVE_ROM) {
        MessageBox(NULL,
                "File 'exec.bin' does not contain the correct Executive ROM.",
                "Error", MB_OK);
        return -1;
    }

    //load the executive rom file
    INT32 size;
    ROM::loadROMImage(&execImage, &size, nextFile, TRUE);

    //TODO: verify that the grom rom file exists
    strcpy(nextFile, currentDir);
    strcat(nextFile, "\\");
    strcat(nextFile, FILENAME_GROM);
    //verify the CRC of the grom rom file
    crc = CRC32::getCrc(nextFile);
    if (crc != CRC_GROM) {
        MessageBox(NULL,
                "File 'grom.bin' does not contain the correct GROM.",
                "Error", MB_OK);
        delete[] execImage;
        return -1;
    }

    //load the grom rom file
    ROM::loadROMImage(&gromImage, &size, nextFile, FALSE);

    //TODO: check to see if the intellivoice rom file exists
    strcpy(nextFile, currentDir);
    strcat(nextFile, "\\");
    strcat(nextFile, FILENAME_ECS_ROM);
    //verify the CRC of the Intellivoice rom file
    crc = CRC32::getCrc(nextFile);
    if (crc == CRC_ECS_ROM) {
        //load the ECS rom file
        ROM::loadROMImage(&ecsImage, &size, nextFile, TRUE);
    }

    //TODO: check to see if the intellivoice rom file exists
    strcpy(nextFile, currentDir);
    strcat(nextFile, "\\");
    strcat(nextFile, FILENAME_IVOICE_ROM);
    //verify the CRC of the Intellivoice rom file
    crc = CRC32::getCrc(nextFile);
    if (crc == CRC_IVOICE_ROM) {
        //load the intellivoice ROM file
        ROM::loadROMImage(&ivoiceImage, &size, nextFile, FALSE);
    }

    //scan the ROM directory
    scanROMDirectory();
/*
XanaduMainWindow window;

MSG msg; 
while( GetMessage( &msg, NULL, 0, 0 ) )
{
    TranslateMessage( &msg );
    DispatchMessage( &msg );
}

return msg.wParam;
*/

    if (strlen(lpCmdLine) > 0) {
        //run a single-player game and exit
        crc = CRC32::getCrc(lpCmdLine);
        CartridgeType* cartType = CartridgeTypeFactory::FACTORY.
                getCartridgeType(crc);
        if (cartType == NULL) {
            MessageBox(NULL, "File does not contain a known cartridge", "Error",
                    MB_OK);
            return -1;
        }

        Cartridge* cart = Cartridge::loadCartridge(lpCmdLine);
        playCartridge(cart);
        delete cart;
    }
    else {
        //run a Kaillera-enabled multiplayer game

        //determine the length of the string needed to hold the game list
        //we'll need to give to Kaillera
        INT32 lsize = Xanadu::cartridgeFileList.size();
        INT32 length = 0;
        for (INT32 i = 0; i < lsize; i++) {
            if (Xanadu::cartridgeFileList[i]->cartType->requiresECS() &&
                    Xanadu::ecsImage == NULL)
                continue;
            length += strlen(Xanadu::cartridgeFileList[i]->cartType->getName()) + 1;
        }
    
        //now copy the list into the string
        kinfos.gameList = new char[length+1];
        char* currentP = kinfos.gameList;
        INT32 nextPos = 0;
        for (i = 0; i < lsize; i++) {
            if (Xanadu::cartridgeFileList[i]->cartType->requiresECS() &&
                    Xanadu::ecsImage == NULL)
                continue;

            const char* nextString = Xanadu::cartridgeFileList[i]->cartType->getName();
            strcpy(&currentP[nextPos], nextString);
            nextPos += strlen(nextString);
            currentP[nextPos] = NULL;
            nextPos++;
        }
        currentP = NULL;
    
        InitCommonControls();
        kailleraInit();
        kinfos.appName = "Xanadu";
        kinfos.gameCallback = startGame;
        kailleraSetInfos(&kinfos);
        kailleraSelectServerDialog(NULL);
        kailleraShutdown();
    }

    return 0;
}

void Xanadu::scanROMDirectory()
{
    clearCartList();

    //scan for the new list
    WIN32_FIND_DATA nextFile;
    HANDLE findHandle = FindFirstFile("roms\\*", &nextFile);
    if (findHandle == INVALID_HANDLE_VALUE)
        return;

    char nextPath[MAX_PATH];
    do {
        sprintf(nextPath, "roms\\%s", nextFile.cFileName);
        UINT32 crc = CRC32::getCrc(nextPath);
        CartridgeType* nextCartType = CartridgeTypeFactory::FACTORY.
                getCartridgeType(crc);
        if (nextCartType != NULL) {
            CartridgeFile* nextCartFile = new CartridgeFile();
            nextCartFile->cartType = nextCartType;
            strcpy(nextCartFile->filename, nextFile.cFileName);
            Xanadu::cartridgeFileList.push_back(nextCartFile);
        }
    } while (FindNextFile(findHandle, &nextFile));
    FindClose(findHandle);
}

void Xanadu::clearCartList() {
    //delete the current list
    INT32 size = Xanadu::cartridgeFileList.size();
    for (INT32 i = 0; i < size; i++)
        delete Xanadu::cartridgeFileList[i];
    Xanadu::cartridgeFileList.erase(Xanadu::cartridgeFileList.begin(), Xanadu::cartridgeFileList.end());
}

Xanadu::~Xanadu() {
    clearCartList();
    if (kinfos.gameList != NULL)
        delete[] kinfos.gameList;
    //clean up
    if (ivoiceImage != NULL)
        delete[] ivoiceImage;
    if (ecsImage != NULL)
        delete[] ecsImage;
    if (gromImage != NULL)
        delete[] gromImage;
    if (execImage != NULL)
        delete[] execImage;

}

INT32 Xanadu::playCartridge(Cartridge* cart)
{
    Intellivision inty;
    inty.setExecImage(Xanadu::execImage);
    inty.setGROMImage(Xanadu::gromImage);

    Win32PlugIn plugIn;
    ((DirectXVideoOutputDevice*)plugIn.getVideoOutputDevice())->
            setFullScreenMode(TRUE);
/*
*/
    inty.setPlugIn(&plugIn);

    if (cart->requiresECS()) {
        if (Xanadu::ecsImage == NULL) {
            MessageBox(NULL,
                    "ECS support required\nSee 'Readme.txt' for details",
                    "Error", MB_OK);
        }
        inty.enableECSSupport(Xanadu::ecsImage);
    }

    if (cart->usesIntellivoice() && Xanadu::ivoiceImage != NULL) {
        inty.enableIntellivoiceSupport(Xanadu::ivoiceImage);
    }

    //insert the cartridge
    inty.insertCartridge(cart);

    //turn on the Intellivision
    INT32 result;
    if (result = inty.turnOn()) {
        MessageBox(NULL,
                "Unable to initialize DirectX", "Error", MB_OK);
        return -1;
    }

    //run the emulation
    inty.run();

    //turn off the Intellivision
    inty.turnOff();

    return 0;
}

int WINAPI startGame(char* game, int player, int numPlayers)
{
    //get the actual filename from the list of scanned files
    INT32 lsize = Xanadu::cartridgeFileList.size();
    char* filename = NULL;
    for (INT32 i = 0; i < lsize; i++) {
        if (!strcmp(game, Xanadu::cartridgeFileList[i]->cartType->getName())) {
            filename = Xanadu::cartridgeFileList[i]->filename;
            break;
        }
    }

    if (filename == NULL) {
        MessageBox(NULL, "File not found", "Error", MB_OK);
        return -1;
    }

    char filepath[MAX_PATH];
    sprintf(filepath, "roms\\%s", filename);

    //TODO: verify that the specified cartridge file exists verify that the
    //      specified cartridge file contains the correct cartridge
    UINT32 crc = CRC32::getCrc(filepath);
    CartridgeType* cartType = CartridgeTypeFactory::FACTORY.
            getCartridgeType(crc);
    if (cartType == NULL) {
        char msg[256];
        sprintf(msg, "Error loading file '%s'", filepath);
        MessageBox(NULL, msg, "Error", MB_OK);
        return -1;
    }

    //load the cartridge file
    Cartridge* cart = Cartridge::loadCartridge(filepath);

    //create the emulator
    Intellivision inty;
    inty.setExecImage(Xanadu::execImage);
    inty.setGROMImage(Xanadu::gromImage);
    if (Xanadu::ecsImage != NULL)
        inty.enableECSSupport(Xanadu::ecsImage);
    if (Xanadu::ivoiceImage != NULL)
        inty.enableIntellivoiceSupport(Xanadu::ivoiceImage);

    //init the plug-in
    Win32PlugIn plugIn;
    inty.setPlugIn(&plugIn);
    inty.insertCartridge(cart);

    inty.setNetworkGame(TRUE);
    if (player == 1) {
        inty.setPlayersSwitched(FALSE);
        inty.setPlayerOneRemote(FALSE);
        if (numPlayers > 1)
            inty.setPlayerTwoRemote(TRUE);
    }
    else if (player == 2) {
        inty.setPlayersSwitched(TRUE);
        inty.setPlayerOneRemote(TRUE);
        inty.setPlayerTwoRemote(FALSE);
    }
    else {
        inty.setPlayerOneRemote(TRUE);
        inty.setPlayerTwoRemote(TRUE);
    }

    //turn on the Intellivision
    INT32 result;
    if (result = inty.turnOn()) {
        MessageBox(NULL,
                "Unable to initialize DirectX", "Error", MB_OK);
        return -1;
    }

    //run the emulation
    inty.run();

    //turn off the Intellivision
    inty.turnOff();

    //delete the cartridge
    delete cart;

    return 0;
}
