
#include <iostream>
#include "DirectXAudioOutputDevice.h"
#include "DirectXVideoOutputDevice.h"

using namespace std;

extern DirectXVideoOutputDevice* currentDevice;

DirectXAudioOutputDevice::DirectXAudioOutputDevice() {
    dsound = NULL;
    primarySoundBuffer = NULL;
    secondarySoundBuffer = NULL;
}

INT32 DirectXAudioOutputDevice::init(Intellivision* inty) {
    if (FAILED(DirectSoundCreate8(&DSDEVID_DefaultPlayback, &dsound, NULL))) {
        return -1;
    }

    if (FAILED(dsound->SetCooperativeLevel(currentDevice->m_hWnd,
            DSSCL_NORMAL)))
    {
        return -1;
    }

    DSBUFFERDESC bufferDesc;
    bufferDesc.dwSize = sizeof(DSBUFFERDESC);
    bufferDesc.dwFlags = DSBCAPS_PRIMARYBUFFER;
    bufferDesc.dwBufferBytes = 0;
    bufferDesc.dwReserved = 0;
    bufferDesc.lpwfxFormat = NULL;
    bufferDesc.guid3DAlgorithm = GUID_NULL;
    if (FAILED(dsound->CreateSoundBuffer(&bufferDesc, &primarySoundBuffer,
            NULL)))
    {
        return -1;
    }

    WAVEFORMATEX wfx;
    wfx.wFormatTag = WAVE_FORMAT_PCM;
    wfx.nChannels = 1;
    wfx.nSamplesPerSec = 22050;
    wfx.wBitsPerSample = 16;
    wfx.nBlockAlign = (wfx.nChannels * wfx.wBitsPerSample)/8;
    wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
    wfx.cbSize = 0;

    bufferDesc.dwFlags = DSBCAPS_GLOBALFOCUS;
    bufferDesc.dwBufferBytes = 8820;
    bufferDesc.dwReserved = 0;
    bufferDesc.lpwfxFormat = &wfx;
    bufferDesc.guid3DAlgorithm = GUID_NULL;
    if (FAILED(dsound->CreateSoundBuffer(&bufferDesc, &secondarySoundBuffer,
            NULL)))
    {
        return -1;
    }

    if (FAILED(primarySoundBuffer->Play(0, 0, DSBPLAY_LOOPING))) {
        return -1;
    }

    if (FAILED(secondarySoundBuffer->Play(0, 0, DSBPLAY_LOOPING))) {
        return -1;
    }

    return 0;
}

INT32 DirectXAudioOutputDevice::getSampleRate() {
    return 22050;
}

UINT8 samples[4410];
UINT32 sampleCount = 0;
UINT32 writePosition = 0;

void DirectXAudioOutputDevice::playSample(INT32 sample) {
    //THIS IS TEMPORARY...
    sample >>= 1;

    samples[sampleCount] = (UINT8)(sample & 0x00FF);
    samples[sampleCount+1] = (UINT8)(((sample & 0x80000000) >> 24) |
            ((sample & 0xFF00) >> 8));
    sampleCount += 2;

    if (sampleCount < 4410)
        return;

    sampleCount = 0;

    DWORD playCursor;
    DWORD writeCursor;
    do {
        if (FAILED(secondarySoundBuffer->GetCurrentPosition(&playCursor,
                &writeCursor)))
        { 
            return;
        }
    }
    while ((playCursor >= writePosition && playCursor <= writePosition+4410) ||
            (writeCursor >= writePosition && writeCursor <= writePosition+4410));

    VOID* ptr1;
    VOID* ptr2;
    DWORD size1;
    DWORD size2;
    if (FAILED(secondarySoundBuffer->Lock(writePosition, 4410,
            &ptr1, &size1, &ptr2, &size2, 0)))
    {
        if (FAILED(secondarySoundBuffer->Restore()))
            return;

        if (FAILED(secondarySoundBuffer->Lock(writePosition, 4410,
                &ptr1, &size1, &ptr2, &size2, 0)))
        {
            return;
        }
    }
    memcpy(ptr1, samples, size1);
    if (4410-size1 > 0)
        memcpy(ptr2, samples+size1, 4410-size1);

    secondarySoundBuffer->Unlock(ptr1, size1, ptr2, 4410-size1);

    if (writePosition == 0)
        writePosition = 4410;
    else
        writePosition = 0;
}

void DirectXAudioOutputDevice::release(Intellivision* inty)
{
    if (secondarySoundBuffer != NULL) {
        secondarySoundBuffer->Stop();
        secondarySoundBuffer->Release();
        secondarySoundBuffer = NULL;
    }
    if (primarySoundBuffer != NULL) {
        primarySoundBuffer->Stop();
        primarySoundBuffer->Release();
        primarySoundBuffer = NULL;
    }
    if (dsound != NULL) {
        dsound->Release();
        dsound = NULL;
    }
}

